/*
 *
 * Copyright (c) 1997-1999 Scott Oaks and Henry Wong. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


public class LoopUser implements ScaleTester {
	private float lookupValues[][];
	private int nRows, nCols, nThreads;
	private LoopUserThread[] worker;

	private class LoopUserThread extends Thread {
		int start, end;
		public LoopUserThread(int start, int end) {
			this.start = start;
			this.end = end;
		}

		public void run() {
			for (int i = start; i < end; i++) {
				for (int j = 1; j < nRows; j++) {
					float sinValue = (float)Math.sin((i % 360)*Math.PI/180.0);
					lookupValues[j][i] = sinValue * (float)i / 180.0f;
					lookupValues[j][i] += lookupValues[j-1][i]*(float)j/180.0f;
				}  
			}
		}
	}

	public void init(int nRows, int nCols, int nThreads) {
		int start, end;

		this.nRows = nRows;
		this.nCols = nCols;
		this.nThreads = nThreads;
		lookupValues = new float[nRows][];
		for (int j = 0; j < nRows; j++) {
			lookupValues[j] = new float[nCols];
		}
		end = 0;
		worker = new LoopUserThread[nThreads];
		for (int i = 0; i < nThreads; i++) {
			start = end;
			end += (nCols / nThreads) + 1;
			if (end > nCols)
				end = nCols;
			worker[i] = new LoopUserThread(start, end);
		}
	}	

	public float[][] doCalc() {
		for (int i = 0; i < nCols; i++) {
			lookupValues[0][i] = 0;
		}
		for (int i = 0; i < nThreads; i++)
			worker[i].start();
		for (int i = 0; i < nThreads; i++) {
			try {
				worker[i].join();
			} catch (InterruptedException ie) {}
		}
		return lookupValues;
	}
}
